@@ -60,15 +60,15 @@ class TestContext(context.RequestContext):
     This is representative of how at least some of our consumers use the
     RequestContext class in their projects.
     """
-    FROM_DICT_EXTRA_KEYS = ['foo']
+    FROM_DICT_EXTRA_KEYS = ['auth_token_info']
 
-    def __init__(self, foo=None, **kwargs):
+    def __init__(self, auth_token_info=None, **kwargs):
         super(TestContext, self).__init__(**kwargs)
-        self.foo = foo
+        self.auth_token_info = auth_token_info
 
     def to_dict(self):
         d = super(TestContext, self).to_dict()
-        d['foo'] = self.foo
+        d['auth_token_info'] = self.auth_token_info
         return d
 
 
@@ -129,7 +129,7 @@ class ContextTest(test_base.BaseTestCase):
             "auth_token": "token1",
             "user": "user1",
             "user_name": "user1_name",
-            "tenant": "tenant1",
+            "project_id": "tenant1",
             "project_name": "tenant1_name",
             "domain": "domain1",
             "domain_name": "domain1_name",
@@ -143,12 +143,13 @@ class ContextTest(test_base.BaseTestCase):
             "request_id": "request1",
             "global_request_id": "req-uuid",
             "resource_uuid": "instance1",
-            "extra_data": "foo"
+            "extra_data": "foo",
+            "system_scope": "all"
         }
         ctx = context.RequestContext.from_dict(dct)
         self.assertEqual(dct['auth_token'], ctx.auth_token)
         self.assertEqual(dct['user'], ctx.user_id)
-        self.assertEqual(dct['tenant'], ctx.project_id)
+        self.assertEqual(dct['project_id'], ctx.project_id)
         self.assertEqual(dct['domain'], ctx.domain_id)
         self.assertEqual(dct['user_domain'], ctx.user_domain_id)
         self.assertEqual(dct['project_domain'], ctx.project_domain_id)
@@ -163,6 +164,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(dct['domain_name'], ctx.domain_name)
         self.assertEqual(dct['user_domain_name'], ctx.user_domain_name)
         self.assertEqual(dct['project_domain_name'], ctx.project_domain_name)
+        self.assertEqual(dct['system_scope'], ctx.system_scope)
 
     def test_from_dict_unknown_keys(self):
         dct = {
@@ -196,15 +198,15 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual("token1", ctx.auth_token)
         self.assertEqual("user2", ctx.user)
         self.assertEqual("project1", ctx.project_name)
-        self.assertIsNone(ctx.tenant)
+        self.assertIsNone(ctx.project_id)
         self.assertFalse(ctx.is_admin)
         self.assertTrue(ctx.read_only)
 
     def test_from_dict_extended(self):
-        initial = TestContext(foo='bar')
+        initial = TestContext(auth_token_info='foo')
         dct = initial.to_dict()
         final = TestContext.from_dict(dct)
-        self.assertEqual('bar', final.foo)
+        self.assertEqual('foo', final.auth_token_info)
         self.assertEqual(dct, final.to_dict())
 
     def test_is_user_context(self):
@@ -221,6 +223,8 @@ class ContextTest(test_base.BaseTestCase):
         user_id = generate_id(user_name)
         project_name = uuid.uuid4().hex
         project_id = generate_id(project_name)
+        domain_name = uuid.uuid4().hex
+        domain_id = generate_id(domain_name)
         user_domain_name = uuid.uuid4().hex
         user_domain_id = generate_id(user_domain_name)
         project_domain_name = uuid.uuid4().hex
@@ -243,6 +247,7 @@ class ContextTest(test_base.BaseTestCase):
             'HTTP_X_AUTH_TOKEN': auth_token,
             'HTTP_X_USER_ID': user_id,
             'HTTP_X_PROJECT_ID': project_id,
+            'HTTP_X_DOMAIN_ID': domain_id,
             'HTTP_X_USER_DOMAIN_ID': user_domain_id,
             'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
             'HTTP_X_ROLES': ','.join(roles),
@@ -270,6 +275,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(user_id, ctx.user_id)
         self.assertEqual(user_name, ctx.user_name)
         self.assertEqual(project_id, ctx.project_id)
+        self.assertEqual(domain_id, ctx.domain_id)
         self.assertEqual(project_name, ctx.project_name)
         self.assertEqual(user_domain_id, ctx.user_domain_id)
         self.assertEqual(user_domain_name, ctx.user_domain_name)
@@ -316,7 +322,7 @@ class ContextTest(test_base.BaseTestCase):
 
         environ = {'HTTP_X_TENANT': value}
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(value, ctx.tenant)
+        self.assertEqual(value, ctx.project_id)
 
         environ = {'HTTP_X_ROLE': value}
         ctx = context.RequestContext.from_environ(environ=environ)
@@ -348,7 +354,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(new, ctx.project_id)
 
         ctx = context.RequestContext.from_environ(environ=environ,
-                                                  tenant=override)
+                                                  project_id=override)
         self.assertEqual(override, ctx.project_id)
 
         environ = {'HTTP_X_TENANT_NAME': old,
@@ -418,7 +424,7 @@ class ContextTest(test_base.BaseTestCase):
         ctx = context.RequestContext(auth_token=auth_token,
                                      user=user_id,
                                      user_name=user_name,
-                                     tenant=project_id,
+                                     project_id=project_id,
                                      project_name=project_name,
                                      domain=domain_id,
                                      domain_name=domain_name,
@@ -452,7 +458,7 @@ class ContextTest(test_base.BaseTestCase):
         d = ctx.to_dict()
         self.assertIn('auth_token', d)
         self.assertIn('user', d)
-        self.assertIn('tenant', d)
+        self.assertIn('project_id', d)
         self.assertIn('domain', d)
         self.assertIn('user_domain', d)
         self.assertIn('project_domain', d)
@@ -470,7 +476,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertNotIn('project_domain_name', d)
 
         self.assertEqual(auth_token, d['auth_token'])
-        self.assertEqual(project_id, d['tenant'])
+        self.assertEqual(project_id, d['project_id'])
         self.assertEqual(domain_id, d['domain'])
         self.assertEqual(user_domain_id, d['user_domain'])
         self.assertEqual(project_domain_id, d['project_domain'])
@@ -488,7 +494,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIn('auth_token', d)
         self.assertEqual(d['auth_token'], '***')
         self.assertIn('user', d)
-        self.assertIn('tenant', d)
+        self.assertIn('project_id', d)
         self.assertIn('domain', d)
         self.assertIn('user_domain', d)
         self.assertIn('project_domain', d)
@@ -512,6 +518,11 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(user_domain_name, d['user_domain_name'])
         self.assertEqual(project_domain_name, d['project_domain_name'])
 
+    def test_auth_token_info_removed(self):
+        ctx = TestContext(auth_token_info={'auth_token': 'topsecret'})
+        d = ctx.get_logging_values()
+        self.assertNotIn('auth_token_info', d)
+
     def test_dict_empty_user_identity(self):
         ctx = context.RequestContext()
         d = ctx.to_dict()
@@ -535,7 +546,7 @@ class ContextTest(test_base.BaseTestCase):
     def test_policy_dict(self):
         user = uuid.uuid4().hex
         user_domain = uuid.uuid4().hex
-        tenant = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
         project_domain = uuid.uuid4().hex
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
         service_user_id = uuid.uuid4().hex
@@ -545,7 +556,7 @@ class ContextTest(test_base.BaseTestCase):
         # default is_admin_project is True
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
-                                     tenant=tenant,
+                                     project_id=project_id,
                                      project_domain=project_domain,
                                      roles=roles,
                                      service_user_id=service_user_id,
@@ -554,7 +565,9 @@ class ContextTest(test_base.BaseTestCase):
 
         self.assertEqual({'user_id': user,
                           'user_domain_id': user_domain,
-                          'project_id': tenant,
+                          'system_scope': None,
+                          'domain_id': None,
+                          'project_id': project_id,
                           'project_domain_id': project_domain,
                           'roles': roles,
                           'is_admin_project': True,
@@ -565,9 +578,61 @@ class ContextTest(test_base.BaseTestCase):
                           'service_roles': service_roles},
                          ctx.to_policy_values())
 
+        # NOTE(lbragstad): This string has special meaning in that the value
+        # ``all`` represents the entire deployment system.
+        system_all = 'all'
+
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     system_scope=system_all,
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': system_all,
+                          'domain_id': None,
+                          'project_id': None,
+                          'project_domain_id': None,
+                          'roles': roles,
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
+        # context representing a domain-scoped token.
+        domain_id = uuid.uuid4().hex
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     domain_id=domain_id,
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': None,
+                          'domain_id': domain_id,
+                          'project_id': None,
+                          'project_domain_id': None,
+                          'roles': roles,
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
-                                     tenant=tenant,
+                                     project_id=project_id,
                                      project_domain=project_domain,
                                      roles=roles,
                                      is_admin_project=False,
@@ -577,7 +642,9 @@ class ContextTest(test_base.BaseTestCase):
 
         self.assertEqual({'user_id': user,
                           'user_domain_id': user_domain,
-                          'project_id': tenant,
+                          'system_scope': None,
+                          'domain_id': None,
+                          'project_id': project_id,
                           'project_domain_id': project_domain,
                           'roles': roles,
                           'is_admin_project': False,
@@ -591,13 +658,13 @@ class ContextTest(test_base.BaseTestCase):
     def test_policy_deprecations(self):
         user = uuid.uuid4().hex
         user_domain = uuid.uuid4().hex
-        tenant = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
         project_domain = uuid.uuid4().hex
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
-                                     tenant=tenant,
+                                     project_id=project_id,
                                      project_domain=project_domain,
                                      roles=roles)
 
@@ -640,11 +707,9 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(0, len(self.warnings))
         self.assertEqual(user_id, ctx.user)
         self.assertEqual(1, len(self.warnings))
-        self.assertEqual(project_id, ctx.tenant)
-        self.assertEqual(2, len(self.warnings))
         self.assertEqual(domain_id, ctx.domain)
-        self.assertEqual(3, len(self.warnings))
+        self.assertEqual(2, len(self.warnings))
         self.assertEqual(user_domain_id, ctx.user_domain)
-        self.assertEqual(4, len(self.warnings))
+        self.assertEqual(3, len(self.warnings))
         self.assertEqual(project_domain_id, ctx.project_domain)
-        self.assertEqual(5, len(self.warnings))
+        self.assertEqual(4, len(self.warnings))
